@@ -1,7 +1,7 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -26,8 +26,9 @@ const char	*str;
  */
 void
 warning(str)
-char	*str;
+const char	*str;
 {
+	warning_str = str;	/* must be a constant string, temporary strings don't work */
 #if	!SILENT
 	if (debug_level >= 0) {
 		set_color(1);	/* set color to yellow */
@@ -48,6 +49,19 @@ error_huge()
 	longjmp(jmp_save, 14);
 }
 
+/*
+ * This is called when a bug test result is positive.
+ *
+ * There is no return.
+ */
+void
+error_bug(str)
+const char	*str;	/* constant string to display */
+{
+	error(str);		/* Display the passed error message. */
+	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
+}
+
 /*
  * Check if a floating point math function flagged an error.
  *
@@ -127,6 +141,8 @@ init_gvars()
 void
 clean_up()
 {
+	int	i;
+
 	init_gvars();		/* reset the global variables to the default */
 	if (gfp != stdout) {	/* reset the output file to standard output */
 #if	!SECURE
@@ -134,6 +150,12 @@ clean_up()
 #endif
 		gfp = stdout;
 	}
+	for (i = 0; i < n_equations; i++) {
+		if (n_lhs[i] <= 0) {
+			n_lhs[i] = 0;
+			n_rhs[i] = 0;
+		}
+	}
 }
 
 /*
@@ -244,14 +266,23 @@ alloc_next_espace()
 {
 	int	i, n;
 
-	for (n = cur_equation, i = 1;; n = (n + 1) % n_equations, i++) {
-		if (i > n_equations) {
-			i = n_equations;
-			if (!alloc_espace(i)) {
+	for (n = cur_equation, i = 0;; n = (n + 1) % N_EQUATIONS, i++) {
+		if (i >= N_EQUATIONS)
+			return -1;
+		if (n >= n_equations) {
+			n = n_equations;
+			if (!alloc_espace(n)) {
+				warning(_("Memory is exhausted."));
+				for (n = 0; n < n_equations; n++) {
+					if (n_lhs[n] == 0) {
+						n_rhs[n] = 0;
+						return n;
+					}
+				}
 				return -1;
 			}
 			n_equations++;
-			return i;
+			return n;
 		}
 		if (n_lhs[n] == 0)
 			break;
@@ -284,10 +315,10 @@ next_espace()
  */
 void
 copy_espace(src, dest)
-int	src, dest;	/* indices */
+int	src, dest;	/* equation space numbers */
 {
 	if (src == dest) {
-		error("Internal error: copy_espace(src, dest) source and destination the same.");
+		error("Internal error: copy_espace() source and destination the same.");
 		return;
 	}
 	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
@@ -313,7 +344,7 @@ int	i;
 }
 
 /*
- * Return true if variable "v" exists in expression.
+ * Return the number of times variable "v" is found in an expression.
  */
 int
 found_var(p1, n, v)
@@ -322,15 +353,16 @@ int		n;	/* expression length */
 long		v;	/* standard Mathomatic variable */
 {
 	int	j;
+	int	count = 0;
 
 	if (v) {
 		for (j = 0; j < n; j++) {
 			if (p1[j].kind == VARIABLE && p1[j].token.variable == v) {
-				return true;
+				count++;
 			}
 		}
 	}
-	return false;
+	return count;
 }
 
 /*
@@ -338,7 +370,7 @@ long		v;	/* standard Mathomatic variable */
  */
 int
 var_in_equation(i, v)
-int	i;	/* index */
+int	i;	/* equation space number */
 long	v;	/* standard Mathomatic variable */
 {
 	if (n_lhs[i] <= 0)
@@ -447,6 +479,12 @@ int		*np;		/* pointer to the returned parsed expression length */
 	char	buf[DEFAULT_N_TOKENS];
 	char	*cp;
 
+#if	LIBRARY
+	snprintf(buf, sizeof(buf), "#%+d", pull_number + 1);
+	pull_number++;
+	cp = parse_expr(equation, np, buf);
+	return(cp && *np > 0);
+#else
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
@@ -457,6 +495,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 		}
 	}
 	return(*np > 0);
+#endif
 }
 
 /*
@@ -552,7 +591,8 @@ int	n;		/* maximum size of "string" in bytes */
 	if (readline_enabled && !echo_input) {
 		cp = readline(prompt_str);
 		if (cp == NULL) {
-			fprintf(gfp, _("\nEnd of input.\n"));
+			if (!quiet_mode)
+				printf(_("\nEnd of input.\n"));
 			exit_program(0);
 		}
 		my_strlcpy(string, cp, n);
@@ -567,7 +607,8 @@ int	n;		/* maximum size of "string" in bytes */
 			printf("%s", prompt_str);
 		}
 		if (fgets(string, n, stdin) == NULL) {
-			fprintf(gfp, _("\nEnd of input.\n"));
+			if (!quiet_mode)
+				printf(_("\nEnd of input.\n"));
 			exit_program(0);
 		}
 	}
@@ -576,7 +617,8 @@ int	n;		/* maximum size of "string" in bytes */
 		printf("%s", prompt_str);
 	}
 	if (fgets(string, n, stdin) == NULL) {
-		fprintf(gfp, _("\nEnd of input.\n"));
+		if (!quiet_mode)
+			printf(_("\nEnd of input.\n"));
 		exit_program(0);
 	}
 #endif
@@ -586,8 +628,8 @@ int	n;		/* maximum size of "string" in bytes */
 		string[i] = '\0';
 	}
 	if (gfp != stdout || echo_input) {
-		/* Input that is prompted for is now included in the redirected output of a command to a file,
-		   making redirection like logging. */
+		/* Input that is prompted for is now included in the redirected output
+		   of a command to a file, making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
 	}
 	set_error_level(string);
@@ -628,16 +670,14 @@ get_yes_no()
  */
 int
 return_result(en)
-int	en;	/* equation space number */
+int	en;	/* equation space number the result is in */
 {
 	if (n_lhs[en] <= 0) {
 		return false;
 	}
 #if	LIBRARY
 	if (gfp == stdout) {
-		if (display2d) {
-			make_fractions_and_group(en);
-		}
+		make_fractions_and_group(en);
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
@@ -774,8 +814,10 @@ char	*
 skip_space(cp)
 char	*cp;	/* character pointer */
 {
-	while (cp && *cp && isspace(*cp))
-		cp++;
+	if (cp) {
+		while (*cp && isspace(*cp))
+			cp++;
+	}
 	return cp;
 }
 
