@@ -1,7 +1,7 @@
 /*
- * Main include file for Mathomatic, an algebraic manipulator written entirely in C.
+ * Main include file for Mathomatic.  Can be edited.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #ifndef	max
@@ -27,8 +27,8 @@
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N	/* internationalization: allow other languages if all strings are translated; not tested */
-#define _(str)		gettext(str)
+#if	I18N		/* internationalization: output in languages other than English if strings are translated */
+#define _(str)		gettext(str)	/* currently incomplete and untested */
 #else
 #define _(str)		str
 #endif
@@ -41,17 +41,20 @@
 #define	PROMPT		"-> "				/* user interface prompt string */
 #define	HTML_PROMPT	"&mdash;&gt; "
 
-#define	MAX_CMD_LEN	PATH_MAX			/* maximum command line length (not equations), also max file name length */
+#define	MAX_CMD_LEN	(max(PATH_MAX, 500))		/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
  * The following defines the maximum number of equation spaces that can be allocated.
+ * This affects maximum memory usage.
  */
+#ifndef	N_EQUATIONS
 #if	HANDHELD
 #define	N_EQUATIONS	20
 #else
 #define	N_EQUATIONS	100
 #endif
+#endif
 
 /*
  * The following defines the default maximum mathematical expression size.
@@ -61,11 +64,13 @@
  * This should be made much smaller for handhelds and embedded systems.
  * Do not set to less than 100.
  */
+#ifndef	DEFAULT_N_TOKENS
 #if	HANDHELD
 #define	DEFAULT_N_TOKENS	300
 #else
 #define	DEFAULT_N_TOKENS	30000
 #endif
+#endif
 
 #define	DIVISOR_SIZE		(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
@@ -102,6 +107,7 @@ typedef union {
 #define	IMAGINARY	3L	/* the imaginary constant "i" or "i#" */
 #define	SIGN		4L	/* for two-valued "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
+#define	V_INTEGER_NAME	"integer"	/* name for the integer variable type */
 	int	operatr;	/* internal storage for Mathomatic operators */
 /* Valid operators follow (order doesn't matter), 0 is reserved for no operator: */
 #define	PLUS		1	/* a + b */
