@@ -1,7 +1,7 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -19,16 +19,6 @@ static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
 #endif
 
-/*
- * Compare function for qsort(3).
- */
-static int
-vcmp(p1, p2)
-sort_type	*p1, *p2;
-{
-	return(p2->count - p1->count);
-}
-
 /*
  * The version command.
  */
@@ -38,6 +28,7 @@ char	*cp;	/* the command line argument */
 {
 	int		rv;	/* return value */
 
+	extra_characters(cp);	/* no command options allowed */
 #if	LIBRARY
 	free(result_str);
 	result_str = strdup(VERSION);
@@ -81,7 +72,8 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, _("\nMaximum memory usage: %ld kilobytes\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
+	fprintf(gfp, _("\nThe current expression array size is %d tokens,\n"), n_tokens);
+	fprintf(gfp, _("maximum memory usage: %ld kilobytes.\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
 	fprintf(gfp, _("This is GNU LGPL version 2.1 licensed software.\n"));
 	return true;
 }
@@ -95,7 +87,7 @@ char	*cp;
 {
 	int		i, j;
 	char		buf[MAX_CMD_LEN];
-	long		v;	/* variable */
+	long		v;	/* Mathomatic variable */
 	token_type	want;
 	int		diff_sign;
 	int		verify_flag = false, plural_flag = false, once_through;
@@ -212,7 +204,7 @@ char	*cp;		/* the command line */
 int	product_flag;	/* true for product, otherwise sum */
 {
 	int		i;
-	long		v = 0;			/* variable */
+	long		v = 0;			/* Mathomatic variable */
 	double		start, end, step = 1.0;
 	int		result_en;
 	int		n, ns;
@@ -264,10 +256,14 @@ int	product_flag;	/* true for product, otherwise sum */
 			return false;
 	}
 	start = strtod(cp1, &cp);
-	if (cp1 == cp || fabs(start) >= MAX_K_INTEGER) {
+	if (cp1 == cp) {
 		error(_("Invalid number."));
 		return false;
 	}
+	if (fabs(start) >= MAX_K_INTEGER) {
+		error(_("Number too large."));
+		return false;
+	}
 	cp = skip_space(cp);
 	if (strcmp_tospace(cp, "to") == 0) {
 		cp = skip_param(cp);
@@ -280,10 +276,14 @@ int	product_flag;	/* true for product, otherwise sum */
 			return false;
 	}
 	end = strtod(cp1, &cp);
-	if (cp1 == cp || fabs(end) >= MAX_K_INTEGER) {
+	if (cp1 == cp) {
 		error(_("Invalid number."));
 		return false;
 	}
+	if (fabs(end) >= MAX_K_INTEGER) {
+		error(_("Number too large."));
+		return false;
+	}
 	cp = skip_space(cp);
 	if (*cp) {
 		cp1 = cp;
@@ -416,8 +416,8 @@ int		*np;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;	/* found sub-expression flags */
-	long	v;			/* variable */
-	char	temp_buf[50];
+	long	v;			/* Mathomatic variable */
+	char	var_name_buf[MAX_VAR_LEN];
 
 	if (*np <= 0) {
 		return false;
@@ -448,8 +448,8 @@ int		*np;
 					k1 = i1 - 1;
 					if ((jj1 - k1) >= OPT_MIN_SIZE
 					    && se_compare(&equation[k], j - k, &equation[k1], jj1 - k1, &diff_sign)) {
-						snprintf(temp_buf, sizeof(temp_buf), "temp%d", last_temp_var);
-						if (parse_var(&v, temp_buf) == NULL) {
+						snprintf(var_name_buf, sizeof(var_name_buf), "temp%d", last_temp_var);
+						if (parse_var(&v, var_name_buf) == NULL) {
 							return false;	/* can't create "temp" variable */
 						}
 						last_temp_var++;
@@ -614,7 +614,7 @@ char	*cp;
 			}
 		}
 	}
-	debug_string(0, _("Expression(s) pushed.  Press the UP key to access."));
+	debug_string(0, _("Expression pushed.  Press the UP key to access."));
 	return true;
 }
 
@@ -663,72 +663,75 @@ display_current_directory()
  * Output the current set options in a format suitable for reading back in.
  */
 void
-output_options()
+output_options(ofp)
+FILE	*ofp;	/* Output file pointer */
 {
-	fprintf(gfp, "precision = %d digits\n", precision);
+	fprintf(ofp, "precision = %d digits\n", precision);
 
 	if (!autosolve) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "autosolve\n");
+	fprintf(ofp, "autosolve\n");
 
 	if (!autocalc) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "autocalc\n");
+	fprintf(ofp, "autocalc\n");
 
 #if	!SILENT
-	fprintf(gfp, "debug_level = %d\n", debug_level);
+	fprintf(ofp, "debug_level = %d\n", debug_level);
 #endif
 
 	if (!case_sensitive_flag) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "case_sensitive\n");
+	fprintf(ofp, "case_sensitive\n");
 
 	if (!color_flag) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "color\n");
+	fprintf(ofp, "color\n");
 
 	if (!bold_colors) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "bold_colors\n");
+	fprintf(ofp, "bold_colors\n");
 
 	if (!display2d) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "display2d\n");
+	fprintf(ofp, "display2d\n");
 
 	if (quiet_mode) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "prompt\n");
+	fprintf(ofp, "prompt\n");
 
 	if (!preserve_surds) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "preserve_surds\n");
+	fprintf(ofp, "preserve_surds\n");
 
-	if (!true_modulus) {
-		fprintf(gfp, "no ");
-	}
-	fprintf(gfp, "true_modulus\n");
+	fprintf(ofp, "modulus_mode = %d\n", modulus_mode);
 
-	fprintf(gfp, "finance = %d\n", finance_option);
+	fprintf(ofp, "finance = %d\n", finance_option);
 
 	if (!factor_int_flag) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "factor_integers\n");
+	fprintf(ofp, "factor_integers\n");
 
 	if (!right_associative_power) {
-		fprintf(gfp, "no ");
+		fprintf(ofp, "no ");
 	}
-	fprintf(gfp, "right_associative_power\n");
+	fprintf(ofp, "right_associative_power\n");
 
-	fprintf(gfp, "special_variable_characters = %s\n", special_variable_characters);
+	if (!negate_highest_precedence) {
+		fprintf(ofp, "no ");
+	}
+	fprintf(ofp, "negate_highest_precedence\n");
+
+	fprintf(ofp, "special_variable_characters = %s\n", special_variable_characters);
 }
 
 /*
@@ -869,9 +872,23 @@ char	*cp;
 		preserve_surds = !negate;
 		goto check_return;
 	}
-	if (strncasecmp(cp, "true_modulus", 10) == 0) {
+	if (strncasecmp(cp, "modulus_mode", 3) == 0) {
 		cp = skip_param(cp);
-		true_modulus = !negate;
+		if (negate) {
+			modulus_mode = 0;
+		} else {
+			i = decstrtol(cp, &cp1);
+			if (cp == cp1 || i < 0 || i > 2) {
+				error(_("Please specify the modulus mode number (0, 1, or 2)."));
+				printf(_("0 means modulus operator (%%) result has same sign as dividend,\n"));
+				printf(_("1 means computed result always has same sign as the divisor,\n"));
+                		printf(_("2 means the result is always positive or zero.\n"));
+				printf(_("Current value is %d.\n"), modulus_mode);
+				return false;
+			}
+			cp = cp1;
+			modulus_mode = i;
+		}
 		goto check_return;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
@@ -900,8 +917,8 @@ char	*cp;
 					return false;
 				}
 			}
-			if (i < 0 || i > 50) {
-				error(_("Minimum is 0, maximum is 50."));
+			if (i < 0 || i > 14) {
+				error(_("Minimum is 0, maximum is 14."));
 				return false;
 			}
 			cp = cp1;
@@ -919,6 +936,11 @@ char	*cp;
 		right_associative_power = !negate;
 		goto check_return;
 	}
+	if (strncasecmp(cp, "negate_highest_precedence", 6) == 0) {
+		cp = skip_param(cp);
+		negate_highest_precedence = !negate;
+		goto check_return;
+	}
 	error(_("Unknown set option."));
 	return false;
 
@@ -938,7 +960,7 @@ char	*cp;
 	if (*cp == '\0') {
 		fprintf(gfp, _("Options are set as follows:\n\n"));
 
-		output_options();
+		output_options(gfp);
 
 		fprintf(gfp, "columns = %d\n", screen_columns);
 
@@ -966,14 +988,17 @@ int
 pause_cmd(cp)
 char	*cp;
 {
+#if	LIBRARY
+	return true;
+#else
 	char	*cp1;
 	char	buf[MAX_CMD_LEN];
 
-	if (test_mode /* || !isatty(0) */) {
+	if (test_mode || !isatty(0)) {
 		return true;
 	}
 	if (*cp == '\0') {
-		cp = _("Please press Enter");
+		cp = _("Please press the Enter key");
 	}
 	snprintf(prompt_str, sizeof(prompt_str), " ==== %s ==== ", cp);
 	if ((cp1 = get_string(buf, sizeof(buf))) == NULL) {
@@ -986,6 +1011,7 @@ char	*cp;
 		return false;
 	}
 	return true;
+#endif
 }
 
 /*
@@ -1032,11 +1058,10 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 {
 	int		i, j, k;
 	int		beg;
-	int		found_imag;
-	int		has_imag, has_real;
+	int		found_imag, has_imag, has_real;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
-	long		v;			/* variable */
+	long		v = IMAGINARY;			/* separation variable */
 
 	if (current_not_defined()) {
 		return false;
@@ -1054,7 +1079,6 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		dest = lhs[j];
 		np = &n_lhs[j];
 	}
-	v = IMAGINARY;
 	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
@@ -1073,8 +1097,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	dest[0] = zero_token;
 	has_imag = has_real = false;
 	for (beg = k = 0; beg < *nps; beg = k, k++) {
-		found_imag = false;
-		for (; k < *nps; k++) {
+		for (found_imag = false; k < *nps; k++) {
 			if (source[k].level == 1 && source[k].kind == OPERATOR
 			    && (source[k].token.operatr == PLUS || source[k].token.operatr == MINUS)) {
 				break;
@@ -1406,6 +1429,11 @@ calc_again:
 			fprintf(gfp, " = ");
 		}
 		list_factor(tlhs, &n_tlhs, factor_flag);
+		if (make_fractions(tlhs, &n_tlhs)) {
+			group_proc(tlhs, &n_tlhs);
+			fprintf(gfp, " = ");
+			list_factor(tlhs, &n_tlhs, factor_flag);
+		}
 		fprintf(gfp, "\n\n");
 	}
 	if (value_entered && repeat_flag)
@@ -1654,7 +1682,7 @@ char	*cp;
 	long		v = 0, v_tmp;		/* Mathomatic variables */
 	int		i, j;
 	int		nl, nr;
-	double		d1, d2, d3, d4, d5;
+	double		lcm, d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
 	if (*cp) {
@@ -1699,16 +1727,17 @@ char	*cp;
 			return true;
 		}
 		fprintf(gfp, "GCD = ");
-		if (factor_one(d3)) {
+		if (factor_one(d3) && !is_prime()) {
 			display_unique();
 		} else {
 			fprintf(gfp, "%.*g\n", precision, d3);
 		}
+		lcm = (d1 * d2) / d3;
 		fprintf(gfp, "LCM = ");
-		if (factor_one((d1 * d2) / d3)) {
+		if (factor_one(lcm) && !is_prime()) {
 			display_unique();
 		} else {
-			fprintf(gfp, "%.*g\n", precision, (d1 * d2) / d3);
+			fprintf(gfp, "%.*g\n", precision, lcm);
 		}
 		return true;
 	}
@@ -1743,7 +1772,7 @@ char	*cp;
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
-		fprintf(gfp, _("Polynomial GCD (iterations = %d):\n"), j);
+		fprintf(gfp, _("Polynomial GCD (%d Euclidean algorithm iteration%s):\n"), j, (j == 1) ? "" : "s");
 		display_fractions_and_group(trhs, &n_trhs);
 		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
@@ -1893,23 +1922,30 @@ int	i;	/* equation number */
 long	v;	/* Mathomatic variable */
 {
 	token_type	want;
+	int		solved;
 
 	if (i == cur_equation) {
 		error(_("Error: source and destination are the same."));
 		return false;
 	}
+	solved = (solved_equation(i) && lhs[i][0].token.variable == v);
 #if	!SILENT
-	if (debug_level >= 0) {
-		list_var(v, 0);
+	list_var(v, 0);
+	if (solved) {
+		/* already solved */
+		printf(_("Substituting the RHS of equation #%d into the current equation for variable (%s)...\n"), i + 1, var_str);
+	} else {
 		printf(_("Solving equation #%d for (%s) and substituting into the current equation...\n"), i + 1, var_str);
 	}
 #endif
-	want.level = 1;
-	want.kind = VARIABLE;
-	want.token.variable = v;
-	if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
-		error(_("Solve failed."));
-		return false;
+	if (!solved) {
+		want.level = 1;
+		want.kind = VARIABLE;
+		want.token.variable = v;
+		if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
+			error(_("Solve failed."));
+			return false;
+		}
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
 	subst_var_with_exp(lhs[cur_equation], &n_lhs[cur_equation], rhs[i], n_rhs[i], v);
@@ -2075,6 +2111,16 @@ char	*cp;
 	return displayed;
 }
 
+/*
+ * Compare function for qsort(3).
+ */
+static int
+vcmp(p1, p2)
+sort_type	*p1, *p2;
+{
+	return(p2->count - p1->count);
+}
+
 /*
  * The variables command.
  */
@@ -2107,7 +2153,7 @@ char	*cp;
 		return false;
 	}
 	last_v = 0;
-	for (vc = 0;; vc++) {
+	for (vc = 0;;) {
 		if (vc >= ARR_CNT(va)) {
 			error(_("Too many variables to list."));
 			return false;
@@ -2147,6 +2193,7 @@ char	*cp;
 		last_v = v1;
 		va[vc].v = v1;
 		va[vc].count = cnt;
+		vc++;
 	}
 	if (vc <= 0)
 		return true;
@@ -2340,7 +2387,7 @@ char	*cp;
 			sign_flag = true;
 			continue;
 		}
-		if (strncasecmp(cp, "symbolic", 3) == 0) {
+		if (strncasecmp(cp, "symbolic", 4) == 0) {
 			symb = true;
 			continue;
 		}
@@ -2471,7 +2518,7 @@ char	*cp;
 {
 	int	i, j;
 	int	i1;
-	long	v;				/* variable */
+	long	v;				/* Mathomatic variable */
 	double	d;
 	char	buf[MAX_CMD_LEN];
 	int	valid_range = false, repeat_flag = false, power_flag = false;
@@ -2487,10 +2534,6 @@ char	*cp;
 			cp = skip_param(cp);
 		}
 		valid_range = get_range(&cp, &i, &j);
-		if (!valid_range) {
-			if (*cp == '\0' || power_flag)
-				return false;
-		}
 	}
 	if (!valid_range) {
 		do {
@@ -2511,6 +2554,9 @@ char	*cp;
 					return false;
 				}
 				display_unique();
+				if (is_prime()) {
+					fprintf(gfp, "Prime number!\n");
+				}
 			}
 		} while (repeat_flag);
 		return true;
@@ -2691,7 +2737,7 @@ char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
-		error(_("No read file specified."));
+		error(_("No read file name specified."));
 		return false;
 	}
 	fp = NULL;
@@ -2773,7 +2819,7 @@ char	*cp;
 	FILE	*fp;
 	int	fd;
 	int	rv;
-	char	tmp_file[MAX_CMD_LEN];
+	char	tmp_file[PATH_MAX];
 
 	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
@@ -2856,7 +2902,7 @@ char	*cp;
 
 	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
-		error(_("No save file specified."));
+		error(_("No save file name specified."));
 		return false;
 	}
 #if	!SILENT
