@@ -1,7 +1,7 @@
 /*
  * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -35,7 +35,7 @@ extern int		factor_int_flag;
 extern int		display2d;
 extern int		approximate_roots;
 extern int		preserve_surds;
-extern int		true_modulus;
+extern int		modulus_mode;
 extern volatile int	screen_columns;
 extern volatile int	screen_rows;
 extern int		finance_option;
@@ -44,7 +44,7 @@ extern int		autocalc;
 extern char		special_variable_characters[256];
 extern int		integer_coefficients;
 extern int		right_associative_power;
-extern int		negate_high_precedence;
+extern int		negate_highest_precedence;
 extern int		power_starstar;
 extern int		debug_level;
 extern int		domain_check;
@@ -65,6 +65,8 @@ extern char		prompt_str[MAX_PROMPT_LEN];
 extern char		*dir_path;
 extern char		*prog_name;
 extern char		*last_history_string;
+extern char		*history_filename;
+extern char		history_filename_storage[PATH_MAX];
 
 extern double		unique[];
 extern int		ucnt[];
@@ -77,6 +79,8 @@ extern int		test_mode;
 extern int		quiet_mode;
 extern int		echo_input;
 extern volatile int	abort_flag;
+extern int		pull_number;
 
 extern const char	*error_str;
 extern char		*result_str;
+extern const char	*warning_str;
