@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -28,7 +28,7 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable iterations]",	"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to restart Mathomatic." },
 #if	!LIBRARY
@@ -43,20 +43,20 @@ static com_type com_list[] = {
 #endif
 {	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
 #if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
-{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file." },
+{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"[\"repeat\"] variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
 {	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
-{	"factor",	NULL,			factor_cmd,		"[\"numbers\" [integers]] or [equation-number-range] [variables]",	"Factor given integers or factor variables in equation spaces." },
-{	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert expressions with algebraic fractions into a single simple fraction." },
+{	"factor",	NULL,			factor_cmd,		"[\"numbers\" [integers]] or [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
+{	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert to a single simple fraction, like \"simplify fraction\" does." },
 {	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help and reference." },
 {	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real." },
 {	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials order times, then simplify." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
 {	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental)." },
 {	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
-#if	!LIBRARY
 {	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule."},
+#if	!LIBRARY
 {	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code." },
 #endif
 #if	!LIBRARY
@@ -96,9 +96,11 @@ static com_type com_list[] = {
 };
 
 /*
- * Process mathematical expression input in Mathomatic, with no solving and no automatic calculation.
+ * Process mathematical expression input in Mathomatic,
+ * with no solving and no automatic calculation.
  *
- * Simply parse the equation or expression text in "cp" and place the result in equation space "n".
+ * Simply parse the equation or expression text in "cp"
+ * and place the result in equation space "n".
  *
  * Return true if successful.
  */
@@ -138,8 +140,16 @@ int	n;
 char	*cp;
 {
 	int	i;
+	char	*cp1, *ep;
+	int	equals_flag = false;
 
-	if (parse_equation(n, cp)) {
+	if ((ep = parse_equation(n, cp))) {
+		for (cp1 = cp; cp1 < ep; cp1++) {
+			if (*cp1 == '=') {
+				equals_flag = true;
+				break;
+			}
+		}
 		if (n_lhs[n] == 0 && n_rhs[n] == 0) {
 			if (strcmp(cp, "=") == 0 && n_lhs[cur_equation] > 0 && n_rhs[cur_equation] > 0) {
 				debug_string(0, _("Swapping sides of the current equation..."));
@@ -155,10 +165,11 @@ char	*cp;
 			return true;
 		}
 		if (n_lhs[n] == 0 || n_rhs[n] == 0) {
-			if (autosolve) {
+			if (autosolve || equals_flag) {
 				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
-				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
-				    || n_rhs[n] == 1) {
+				    || (lhs[n][0].kind == VARIABLE && ((lhs[n][0].token.variable & VAR_MASK) > SIGN || equals_flag))))
+				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
+				    || rhs[n][0].kind == VARIABLE))) {
 					if (solve_espace(n, cur_equation)) {
 						return return_result(cur_equation);
 					}
@@ -174,9 +185,10 @@ char	*cp;
 			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
 				/* easy selecting of equation spaces by just typing in the equation number */
+				cur_equation = lhs[n][0].token.constant - 1;
 				n_lhs[n] = 0;
-				n = lhs[n][0].token.constant - 1;
-				goto return_ok;
+				return_result(cur_equation);
+				return true;
 			}
 #if	!LIBRARY
 			if (autocalc) {
@@ -271,7 +283,8 @@ char	*cp;
 			return parse(i, cp);
 		}
 		cur_equation = i;
-		return return_result(cur_equation);
+		return_result(cur_equation);
+		return true;
 	}
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /* handle shell escape */
@@ -344,6 +357,7 @@ char	*cp;
 #endif
 			remove_trailing_spaces(buf2);
 /* execute the command by calling the command function */
+			pull_number = 0;
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
 			if (fp && gfp != stdout) {
@@ -460,11 +474,6 @@ next_argument:
 				flag = true;
 			}
 		}
-		if (*cp == '!') {
-			P("! - Shell escape, execute shell and any specified system command.");
-			P("Usage: ![system-command]\n");
-			flag = true;
-		}
 		if (flag) {
 			cp1 = cp = skip_space(cp1);
 			while (*cp1 && !isspace(*cp1))
@@ -475,13 +484,11 @@ next_argument:
 			return true;
 		}
 		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
-			P("Report bugs to the author via email:");
-			P("\tGeorge Gesslein II");
-			P("\tgeorgegesslein@gmail.com");
-			P("or on the Ubuntu Launchpad website:");
-			P("\thttp://bugs.launchpad.net/mathomatic\n");
+			P("Report bugs on the Ubuntu Launchpad website:");
+			P("http://launchpad.net/mathomatic\n");
+
+			P("Please include version information from the version command:\n");
 
-			P("Please include the version number:");
 			version_report();
 			return true;
 		}
@@ -503,12 +510,12 @@ next_argument:
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
-			P("    - negate");
+			P("    - negate (highest precedence)");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
 			P("    * multiply        / divide          % modulus         // integral divide");
 			P("    + add             - subtract");
-			P("    = equate\n");
+			P("    = equate (lowest precedence)\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
 			P("Predefined constants and variables follow:\n");
@@ -518,6 +525,7 @@ next_argument:
 			P("    i or i# - the imaginary number (square root of -1)");
 			P("The above constants may also be used most anywhere variables are required.");
 			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1");
+			P("    integer - variable that can only be an integer value");
 			P("    inf - floating point infinity constant (not a variable)\n");
 
 			P("Absolute value notation \"|x|\" and dual polarity \"+/-x\" are understood.");
@@ -534,7 +542,7 @@ next_argument:
 						return false;
 				}
 			}
-			P("Full documentation can be found at \"http://mathomatic.org/math/doc/\".");
+			fprintf(gfp, "End of list; total of %d different commands.\n", ARR_CNT(com_list));
 			return true;
 		}
 		error(_("Unrecognized help topic or command."));
