@@ -1,7 +1,9 @@
 /*
  * Standard include files for Mathomatic.
+ * Automatically includes all necessary include files for
+ * any Mathomatic C source code.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #define	true	1
@@ -12,6 +14,10 @@
 #undef	READLINE
 #endif
 
+#if	__CYGWIN__ && !CYGWIN
+#warning "Compiling under Cygwin without proper defines.  Please define CYGWIN"
+#endif
+
 /* Include files from /usr/include: */
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,6 +43,7 @@
 #endif
 
 /* Include files from the current directory: */
+#include "license.h"	/* the current license for Mathomatic */
 #include "am.h"		/* the main include file for Mathomatic, contains tunable parameters */
 #include "complex.h"	/* floating point complex number arithmetic function prototypes */
 #include "proto.h"	/* global function prototypes */
