@@ -50,7 +50,7 @@ matho_init()
  * if there is no resulting equation or error message.
  */
 int
-matho_process(char *input, char **outputp)
+matho_process(char *input, const char **outputp)
 {
 	int	i;
 	int	rv;
@@ -68,6 +68,7 @@ matho_process(char *input, char **outputp)
 	}
 	result_str = NULL;
 	error_str = NULL;
+	warning_str = NULL;
 	set_error_level(input);
 	if ((rv = process(input))) {
 		*outputp = result_str;
@@ -90,7 +91,7 @@ matho_process(char *input, char **outputp)
  * Returns true if successful.
  */
 int
-matho_parse(char *input, char **outputp)
+matho_parse(char *input, const char **outputp)
 {
 	int	i;
 	int	rv;
@@ -108,6 +109,7 @@ matho_parse(char *input, char **outputp)
 	}
 	result_str = NULL;
 	error_str = NULL;
+	warning_str = NULL;
 	set_error_level(input);
 	i = next_espace();
 #if	true	/* set this true if you want to be able to enter single variable expressions with no solving */
