@@ -15,9 +15,10 @@
 int
 main(int argc, char **argv)
 {
-	char	*cp, *ocp;
-	int	rv;
-	char	buf[1000];
+	char		*cp;		/* character pointer */
+	const char	*ocp;		/* output character pointer */
+	int		rv;		/* return value */
+	char		buf[1000];	/* buffer */
 
 	printf("Mathomatic library test program.\n");
 	if (!matho_init()) {
@@ -34,9 +35,10 @@ main(int argc, char **argv)
 		if (ocp) {
 			printf("%s\n", ocp);
 			if (rv) {
-				free(ocp);
+				free((void *) ocp);
 			}
 		}
 	}
+	printf("\nEOF\n");
 	exit(0);
 }
