@@ -2,7 +2,7 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -15,18 +15,20 @@
 static int	flist();
 static int	flist_recurse();
 
-/* ANSI terminal color code array for 8 color ANSI, we don't use black */
+/* ANSI terminal color code array for 8 color ANSI, we don't use black or white */
+/* because the background may be the same color. */
 static int	carray[] = {
 	32,	/* green (default color) */
 	33,	/* yellow */
 	31,	/* red */
 	35,	/* magenta */
 	34,	/* blue */
-	36,	/* cyan */
-	37	/* white */
+	36	/* cyan */
 };
 
-/* Bright HTML color array */
+/* Bright HTML color array. */
+/* Used when HTML output and "set color" and "set bold" options are enabled. */
+/* Good looking with a dark background. */
 static char	*bright_html_carray[] = {
 	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
@@ -37,7 +39,8 @@ static char	*bright_html_carray[] = {
 	"#0000FF"
 };
 
-/* Dim HTML color array */
+/* Dim HTML color array for color HTML output. */
+/* Good looking with a white background. */
 static char	*html_carray[] = {
 	"green",	/* green (default color) */
 	"olive",
@@ -138,15 +141,11 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-	if (high_prec || export_flag) {
 #if	CYGWIN
-		fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
+	fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
-		fprintf(gfp, "\n");
+	fprintf(gfp, "\n");
 #endif
-	} else {
-		fprintf(gfp, "\n\n");
-	}
 	return len;
 }
 
@@ -162,18 +161,15 @@ int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return 0;
+	make_fractions_and_group(n);
+	if (factor_int_flag) {
+		factor_int_sub(n);
+	}
 	if (display2d) {
 		/* display in fraction format */
-		make_fractions_and_group(n);
-		if (factor_int_flag) {
-			factor_int_sub(n);
-		}
 		return flist_equation(n);
 	} else {
 		/* display in single-line format */
-		if (factor_int_flag) {
-			factor_int_sub(n);
-		}
 		return list1_sub(n, false);
 	}
 }
@@ -217,13 +213,12 @@ long	v;		/* variable to convert */
 int	lang_code;	/* language code */
 {
 	int		j;
-	char		*cp, buf[100];
+	char		*cp = NULL, buf[100];
 
-	cp = NULL;
+	var_str[0] = '\0';
 	switch (v & VAR_MASK) {
 	case V_NULL:
-		cp = "null";
-		break;
+		return(0);
 	case SIGN:
 		cp = "sign";
 		break;
@@ -302,14 +297,14 @@ int	lang_code;	/* language code */
 	}
 	if (cp) {
 		my_strlcpy(var_str, cp, sizeof(var_str));
+		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
+		if (j) {
+			snprintf(buf, sizeof(buf), "%d", j - 1);
+			strcat(var_str, buf);
+		}
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
-	j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
-	if (j) {
-		snprintf(buf, sizeof(buf), "%d", j - 1);
-		strcat(var_str, buf);
-	}
 	return(strlen(var_str));
 }
 
@@ -685,16 +680,14 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 								if (int_flag || language > 1)
 									break;
 								if (language) {
-									cp = "Math.pow";
+									len += fprintf(gfp, "Math.pow");
 								} else {
-									cp = "pow";
+									len += fprintf(gfp, "pow");
 								}
-								len += fprintf(gfp, cp);
 							}
 							break;
 						case FACTORIAL:
-							cp = "fact";
-							len += fprintf(gfp, cp);
+							len += fprintf(gfp, "fact");
 							break;
 						}
 						break;
