@@ -1,8 +1,8 @@
 /*
  * This file contains main() and the startup code for Mathomatic,
- * which is an algebraic manipulator written entirely in C.
+ * which is a small computer algebra system written entirely in C.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  *
  * Output to stderr is only done in this file.  The rest of the Mathomatic code
  * should not output to stderr; error messages should use error() or go to stdout.
@@ -31,6 +31,10 @@
  * Most of the expression manipulation and comparison routines are recursive,
  * calling themselves for each level of parentheses.
  *
+ * Note that equation space numbers internally are 1 less than the equation
+ * space numbers displayed.  That is because internal equation space numbers
+ * are origin 0 array indexes, while displayed equation numbers are origin 1.
+ *
  * See the file "am.h" to start understanding the Mathomatic code and
  * to adjust memory usage.
  */
@@ -45,10 +49,30 @@
 #include <fenv.h>
 #endif
 
-static void usage();
-static int set_signals();
+static char	rc_file[PATH_MAX];
 
-static char	rc_file[MAX_CMD_LEN];
+/*
+ * Display invocation usage info.
+ */
+void
+usage()
+{
+	printf(_("\nMathomatic version %s computer algebra system\n\n"), VERSION);
+	printf(_("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
+	printf(_("Options:\n"));
+	printf(_("  -b            Enable bold color mode.\n"));
+	printf(_("  -c            Toggle color mode.\n"));
+	printf(_("  -h            Display this help and exit.\n"));
+	printf(_("  -m number     Specify a memory size multiplier.\n"));
+	printf(_("  -q            Set quiet mode (don't display prompts).\n"));
+	printf(_("  -r            Disable readline.\n"));
+	printf(_("  -t            Set test mode.\n"));
+	printf(_("  -u            Set unbuffered output.\n"));
+	printf(_("  -v            Display version information, then exit.\n"));
+	printf(_("  -w            Wide output mode, sets unlimited width.\n"));
+	printf(_("  -x            Enable HTML/XHTML output mode.\n"));
+	printf(_("\nPlease refer to the %s man page for more information.\n"), prog_name);
+}
 
 int
 main(argc, argv)
@@ -61,12 +85,9 @@ char	**argv;
 	int		i;
 	char		*cp = NULL;
 	double		numerator, denominator;
-	double		multiplier;
-	int		coption = false;
+	double		new_size;
+	int		coption = false, boption = false, wide_flag = false;
 
-#if	UNIX
-	prog_name = strdup(basename(argv[0]));		/* set prog_name to this executable's filename */
-#endif
 #if	CYGWIN
 	dir_path = strdup(dirname_win(argv[0]));	/* set dir_path to this executable's directory */
 #endif
@@ -74,27 +95,38 @@ char	**argv;
 	init_gvars();
 	gfp = stdout;
 #if	READLINE
+	if ((cp = getenv("HOME")) && prog_name) {
+		snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
+		history_filename = history_filename_storage;
+	}
 	using_history();		/* initialize readline history */
 	stifle_history(500);		/* maximum of 500 entries */
 	rl_inhibit_completion = true;	/* turn off readline file name completion */
 #endif
 
 	/* process command line options */
-	while ((i = getopt(argc, argv, "qrtchuvm:")) >= 0) {
+	while ((i = getopt(argc, argv, "bqrtchuvwxm:")) >= 0) {
 		switch (i) {
+		case 'w':
+			wide_flag = true;
+			break;
+		case 'b':
+			boption = true;
+			break;
 		case 'c':
 			coption++;
 			break;
-		case 'h':
+		case 'x':
 			html_flag = true;
+			wide_flag = true;
 			break;
 		case 'm':
-			multiplier = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
-			if (cp == NULL || *cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
+			new_size = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
+			if (cp == NULL || *cp || new_size <= 0 || new_size >= (INT_MAX / 3)) {
 				fprintf(stderr, _("%s: Invalid memory size multiplier specified.\n"), prog_name);
 				exit(2);
 			}
-			n_tokens = (int) multiplier;
+			n_tokens = (int) new_size;
 			break;
 		case 'q':
 			quiet_mode = true;
@@ -104,26 +136,27 @@ char	**argv;
 			break;
 		case 't':
 			readline_enabled = false;
+			wide_flag = true;
 			test_mode = true;
 			break;
 		case 'u':
 			setbuf(stdout, NULL);	/* make output unbuffered */
 			setbuf(stderr, NULL);
 			break;
+		case 'h':
+			usage();
+			exit(0);
 		case 'v':
 			version_report();
 			exit(0);
 		default:
 			usage();
+			exit(2);
 		}
 	}
-	if (test_mode || html_flag) {
-		screen_columns = 0;
-		screen_rows = 0;
-#if	UNIX || CYGWIN
-	} else {
-		get_screen_size();
-#endif
+	if (n_tokens < 100) {
+		fprintf(stderr, _("%s: Improper equation space size.\n"), prog_name);
+		exit(2);
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
@@ -131,7 +164,7 @@ char	**argv;
 	}
 #if	READLINE && !SECURE
 	if (readline_enabled) {
-		read_history(NULL);	/* restore readline history of previous session */
+		read_history(history_filename);	/* restore readline history of previous session */
 	}
 #endif
 	if (html_flag) {
@@ -141,9 +174,9 @@ char	**argv;
 #if	SECURE
 		printf(_("Secure "));
 #endif
-		printf(_("This is Mathomatic version %s (www.mathomatic.org).\n"), VERSION);
-		printf(_("Copyright (C) 1987-2008 George Gesslein II.\n"));
-		printf(_("%d equation spaces available, %ldK per equation space.\n"),
+		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
+		printf(_("Copyright (C) 1987-2009 George Gesslein II.\n"));
+		printf(_("%d equation spaces available, %ld kilobytes per equation space.\n"),
 		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 	}
 #if	!SECURE
@@ -152,13 +185,30 @@ char	**argv;
 		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, rc_file);
 	}
 #endif
+	if (wide_flag) {
+		screen_columns = 0;
+		screen_rows = 0;
+#if	UNIX || CYGWIN
+	} else {
+		get_screen_size();
+#endif
+	}
 	if (test_mode) {
 		color_flag = false;
 	} else if (coption & 1) {
 		color_flag = !color_flag;
 	}
+	if (boption) {
+		bold_colors = true;
+		color_flag = true;
+	}
 	if (!quiet_mode && color_flag) {
-		printf(_("%s%s color mode enabled.\n"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
+		printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
+		if (!boption) {
+			printf(_("; disable with the -c option.\n"));
+		} else {
+			printf(".\n");
+		}
 	}
 	if ((i = setjmp(jmp_save)) != 0) {
 		/* for error handling */
@@ -171,7 +221,9 @@ char	**argv;
 			break;
 		}
 	} else {
-		set_signals();
+		if (!set_signals()) {
+			fprintf(stderr, _("signal(2) setting failed.\n"));
+		}
 		if (!f_to_fraction(0.5, &numerator, &denominator)
 		    || numerator != 1.0 || denominator != 2.0
 		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)
@@ -180,7 +232,7 @@ char	**argv;
 			fprintf(stderr, _("Roots will not work properly.\n"));
 		}
 #if	!SECURE
-		/* read in files on the command line, exit if error */
+		/* read in files specified on the command line, exit if error */
 		for (i = optind; i < argc; i++) {
 			if (!read_cmd(argv[i])) {
 				exit_program(1);
@@ -200,33 +252,12 @@ char	**argv;
 	return(0);	/* so the compiler doesn't complain */
 }
 
-/*
- * Display invocation usage info and exit with error.
- */
-static void
-usage()
-{
-	printf(_("Mathomatic version %s - automatic algebraic manipulator\n\n"), VERSION);
-	printf(_("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
-	printf(_("Options:\n"));
-	printf(_("  -c            Toggle color mode.\n"));
-	printf(_("  -h            Enable HTML output mode.\n"));
-	printf(_("  -m number     Specify a memory size multiplier.\n"));
-	printf(_("  -q            Set quiet mode (don't display prompts).\n"));
-	printf(_("  -r            Disable readline.\n"));
-	printf(_("  -t            Set test mode.\n"));
-	printf(_("  -u            Set unbuffered output.\n"));
-	printf(_("  -v            Display version number, etc., then exit.\n"));
-	printf(_("\nPlease refer to the man page for more information.\n"));
-	exit(2);
-}
-
 /*
  * All signal(2) initialization goes here.
  *
  * Return true on success.
  */
-static int
+int
 set_signals()
 {
 	int	rv = true;
@@ -239,6 +270,8 @@ set_signals()
 #endif
 	if (signal(SIGINT, inthandler) == SIG_ERR)
 		rv = false;
+	if (signal(SIGQUIT, inthandler) == SIG_ERR)
+		rv = false;
 #if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
 		rv = false;
@@ -248,9 +281,6 @@ set_signals()
 		rv = false;
 	alarm(TIMEOUT_SECONDS);
 #endif
-	if (!rv) {
-		perror("signal(2) setting failed");
-	}
 	return rv;
 }
 
@@ -315,7 +345,7 @@ load_rc()
 
 /*
  * Floating point exception handler.
- * I do not know the proper function calls that should be here to make the GNU TRAP_FPERRORS define work.
+ * I do not know the proper function calls that should be here to make the TRAP_FPERRORS define work.
  * Currently it goes in an endless loop when an FP exception occurs and TRAP_FPERRORS is true,
  * because when this function returns, the exception immediately happens again.
  * Therefore, do not define TRAP_FPERRORS!
@@ -326,15 +356,11 @@ int	sig;
 {
 #if	TRAP_FPERRORS
 	warning(_("Floating point exception."));
-/*
-	feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW);
-	signal(SIGFPE, fphandler);
-*/
 #endif
 }
 
 /*
- * Control-C (interrupt) signal handler.
+ * Fancy Control-C (interrupt) signal handler.
  * Interrupts processing and returns to main prompt through a polling mechanism.
  * If it can't, repeated calls terminate this program.
  */
@@ -347,13 +373,14 @@ int	sig;
 	case 0:
 	case 1:
 		/* wait for graceful abort */
+		printf(_("\nUser interrupt signal received.\n"));
 		return;
 	case 2:
 		printf(_("\nPress Control-C once more to quit program.\n"));
-		break;
+		return;
 	default:
 		/* abruptly quit this program */
-		printf(_("\nControl-C repeatedly pressed; Quitting...\n"));
+		printf(_("\nRepeatedly interrupted; Quitting...\n"));
 		exit_program(1);
 	}
 }
@@ -402,7 +429,7 @@ int	sig;
 #endif
 
 /*
- * Exit this program and return to the Operating System.
+ * Properly exit this program and return to the operating system.
  */
 void
 exit_program(exit_value)
@@ -414,9 +441,11 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 	}
 #if	READLINE && !SECURE
 	if (readline_enabled) {
-		write_history(NULL);	/* save readline history in ~/.history */
+		write_history(history_filename);	/* save readline history */
 	}
 #endif
+	if (!quiet_mode)
+		printf(_("Thank you for using Mathomatic!\n"));
 	exit(exit_value);
 }
 #endif
