@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -17,9 +17,11 @@ void
 str_tolower(cp)
 char	*cp;
 {
-	for (; cp && *cp; cp++) {
-		if (isupper(*cp))
-			*cp = tolower(*cp);
+	if (cp) {
+		for (; *cp; cp++) {
+			if (isupper(*cp))
+				*cp = tolower(*cp);
+		}
 	}
 }
 
@@ -66,10 +68,9 @@ int		i;	/* location of operator to parenthesize in expression */
 	int	j;
 	int	level;
 
-#if	false
+#if	true
 	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
-		error("Internal error in arguments to binary_parenthesize().");
-		longjmp(jmp_save, 13);
+		error_bug("Internal error in arguments to binary_parenthesize().");
 	}
 #endif
 	level = p1[i].level++;
@@ -104,7 +105,7 @@ int		*np;
 		if (equation[i].token.operatr == NEGATE) {
 			/* change negate operator to -1 times the operand: */
 			equation[i].token.operatr = TIMES;
-			if (negate_high_precedence) {
+			if (negate_highest_precedence) {
 				/* make negate top priority by parenthesizing it first: */
 				binary_parenthesize(equation, *np, i);
 			}	/* otherwise negate has same priority as times/divide */
@@ -167,7 +168,7 @@ int		*np;		/* pointer to expression length */
  * The maximum length that can be parsed is the size of an equation side.
  * Any syntax and other errors are carefully reported and give a NULL return.
  *
- * Returns the new string position or NULL if error.
+ * Returns the new string position, or NULL if error.
  */
 char	*
 parse_section(equation, np, cp)
@@ -175,9 +176,9 @@ token_type	*equation;	/* where the parsed expression is stored (equation side) *
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
 {
-	int		n = 0;		/* position in equation[] */
-	int		cur_level = 1;	/* current level of parentheses */
-	int		operand;	/* flip-flop between operand and operator */
+	int		n = 0;			/* position in equation[] */
+	int		cur_level = 1;		/* current level of parentheses */
+	int		operand = false;	/* flip-flop between operand and operator */
 	char		*cp_start, *cp1;
 	double		d;
 	int		abs_count = 0;
@@ -186,18 +187,28 @@ char		*cp;		/* string to parse */
 	if (cp == NULL)
 		return(NULL);
 	cp_start = cp;
-	operand = false;
 	for (;; cp++) {
+		if (n > (n_tokens - 10)) {
+			error_huge();
+		}
 		switch (*cp) {
 		case ' ':
 		case '\t':
 			continue;
 		case '(':
 		case '{':
-			cur_level++;
 			if (operand) {
+#if	true
+				operand = false;
+				equation[n].level = cur_level;
+				equation[n].kind = OPERATOR;
+				equation[n].token.operatr = TIMES;
+				n++;
+#else
 				goto syntax_error;
+#endif
 			}
+			cur_level++;
 			continue;
 		case ')':
 		case '}':
@@ -216,9 +227,6 @@ char		*cp;		/* string to parse */
 		case '\n':
 			goto p_out;	/* terminator encountered */
 		}
-		if (n > (n_tokens - 6)) {
-			error_huge();
-		}
 		operand = !operand;
 		switch (*cp) {
 		case '|':
@@ -362,7 +370,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && (!negate_high_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
+			if (*cp == '-' && (!negate_highest_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -402,6 +410,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
+			cp1 = cp;
 			if (strncasecmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
 			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
@@ -409,12 +418,15 @@ parse_power:
 				cp += strlen(INFINITY_NAME);
 			} else {
 				equation[n].kind = VARIABLE;
-				cp1 = cp;
 				cp = parse_var(&equation[n].token.variable, cp);
 				if (cp == NULL) {
 					return(NULL);
 				}
 			}
+			if (*cp == '(') {
+				put_up_arrow(cp1 - cp_start, _("Named functions currently not implemented."));
+				return(NULL);
+			}
 			cp--;
 			equation[n].level = cur_level;
 			n++;
@@ -448,7 +460,7 @@ syntax_error:
 /*
  * Parse an equation string into equation space "n".
  *
- * Returns the new string position or NULL on error.
+ * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
  */
 char	*
@@ -472,7 +484,7 @@ char	*cp;	/* pointer to the beginning of the equation character string */
 /*
  * Parse an expression with equation space pull if the line starts with "#" followed by an equation number.
  *
- * Returns the new string position or NULL on error.
+ * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
  */
 char	*
@@ -515,8 +527,10 @@ char		*cp;		/* string to parse */
 			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
 			*np = n_lhs[i];
 		}
+#if	!SILENT
 		list_proc(equation, *np, false);
 		fprintf(gfp, "\n");
+#endif
 		return cp2;
 	}
 	if ((cp = parse_section(equation, np, cp)) != NULL) {
